#
# This file is part of python sample package for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2022 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# Sample to get new broadway2-API extension for Frame-Statistics
# This will work for all USB-Stick on 100BASE-T1 and 10BASE-T1L

# V1.0.0-1  added option -r to resrt counters

import sys, os
import binascii
import platform
import time, datetime
import threading, queue
import binascii

from broadway2 import Broadway, FCAdapter


def main(args):

    reset = False

    try:
        if args[1] == '-r':
            reset = True
    except:
        pass
    
    # create adapter singleton
    base = Broadway()

    print('Connected Devices: ')
    print('='*25)

    aList = base.get_adapters()

    if len(aList) > 1:
        cnt = 1
        for a in aList:
            print('[ {0:2} ]: {1} / {2}'.format(cnt, a[1], a[0]))
            cnt += 1
        try:
            sel = int(input('Enter Number to use:'))
        except:
            sel = 0
        
    else:
        sel = 1

    if sel < 1:
        print('Wrong selection. Exit')
        sys.exit(-1)

    inst, friendlyName = aList[sel-1]    # get adapter selected by instance   
    
    selAdapter = base.get_adapter(inst)
        
    

    devControl = base.open_control(selAdapter.Inst)
    selAdapter.connect(devControl)


    # There are 2 options to get Frame-Statistics

    # read-only:
    # get_frame_stats()
    # 
    #  or read-and-reset 
    # get_frame_stats_reset()


    print()
    if reset:
        stats = selAdapter.get_frame_stats_reset()    # read and reset counters
        print('<<<<<Reset-Option detected. All Statistik-Counters are reset now.>>>>>')
    else:
        stats = selAdapter.get_frame_stats()    # read-only, leave counters running
    
    #stats = selAdapter.get_frame_stats_reset() # read and reset all counters

    print('Adapter: {0} / ({1})'.format(selAdapter.Inst, selAdapter.FriendlyName))
    print('='*48)
    print('Time: {0}.{1}'.format(stats.TimeStampSec, stats.TimeStampNano))
    print('-'*48)
    print('TxFramesTotal: {0}'.format(stats.TxFramesTotal))
    print('RxFramesTotal: {0}'.format(stats.RxFramesTotal))
    
    print()
    print('Errors:')
    print('-'*48)
    print('RxFramesDefective: {0}'.format(stats.RxFramesDefective))
    print('RxFramesCRCError: {0}'.format(stats.RxFramesCrcError))
    print('RxFramesLengthError: {0}'.format(stats.RxFramesLengthError))
    print('RxFramesRejected: {0}'.format(stats.RxFramesRejected))

    print('RxFramesUsbCongestion: {0}'.format(stats.RxFramesUsbCongestion))

    print()
    print('Link-Status:')
    print('-'*48)
    print('LinkUpCount: {0}'.format(stats.get_linkup_count()))
    print('LinkDownCount: {0}'.format(stats.get_linkdown_count()))

    base.close(devControl)

if __name__ == "__main__":
    main(sys.argv)

