#
# This file is part of python sample package for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2019 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import sys
import time
import threading, queue


from broadway2 import broadway_api, Broadway, MultiRawReceiverThread, RawRxHeaderTypes, RawRxFrame


def main(args):
    
    # create adapter singleton
    base = Broadway()

    stop = threading.Event()
    q = queue.Queue()

    multiRawRx = MultiRawReceiverThread(q, stop)

    print('Connected Devices: ')
    print('='*45)

    for a in base.enumerate():
        if a.Features['Raw']:
            dev = base.open_raw_rx(a.Inst, RawRxHeaderTypes.HeaderType2)
            if dev != None: # if dev is used by other program (e.g. streamto) skip it
                multiRawRx.add(a, dev)

                print('found {0}/{1} on {2} and added dev: {3}'.format(a.FriendlyName, a.MACAddress, a.Inst, dev))
            else:
                print('found {0}/{1} on {2} but skipped as busy.'.format(a.FriendlyName, a.MACAddress, a.Inst))

    if multiRawRx.Count < 1:
        return

    multiRawRx.start()

    multiRawRx.rx_enable()

    print('Ctrl-C to exit')

    try:
        while 1:
            try:
                res = q.get(False)  # tuple (devName, rawFrame)
            except:
                res = None
            if res == None:
                time.sleep(0.1)
            else:
                ethRaw = RawRxFrame(1, res[1])
                print('Dev: {0:10} {1}'.format(res[0], ethRaw))
                
                #print('  {0}.{1}.{2} {3}'.format(tgtAddr, srcAddr, ethType, res[3][28:36]))
                    
    except KeyboardInterrupt:

        multiRawRx.rx_disable()

        stop.set()

        for d in multiRawRx.enumerate():
            print('closing: {0}'.format(d))
            base.close(d)
        

if __name__ == "__main__":
    main(sys.argv)
