#
# This file is part of python sample package for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2019 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import sys, os
import binascii
import platform
import time, datetime

from broadway2 import Broadway, broadway_api


DefaultInst = 0 # Use first found USB-Adapter
DefaultPayload = 1024
Delay = 0.2

VLAN_ID=99 # VID=0x38, PCP=0, TCI=0 CFI=0
VLAN_PCP=7
VLAN_TAG=(VLAN_PCP<<13)|VLAN_ID

#VLAN_TAG=-1 # disable


def main(args):

    inst = DefaultInst
    payload = DefaultPayload

    
    # create adapter singleton
    base = Broadway()

    
    aList = base.get_raw_adapters()

    if len(aList) < 1:
        print('No Raw-Adapters found!')
        sys.exit(-1)


    if len(aList) > 1:
        cnt = 1
        for a in aList:
            print('[ {0:2} ]: {1} / {2}'.format(cnt, a[1], a[0]))
            cnt += 1
        try:
            sel = int(input('Enter Number to use:'))
        except:
            sel = 0
        
    else:
        sel = 0


    inst, friendlyName = aList[sel-1]

    adapter = base.get_adapter(inst)

    macSrc = adapter.MACAddress

    print('Open Adapter to send raw: {0} / {1}'.format(friendlyName, inst))    
    print('='*30)
    if sel >= 0:
        devTx = base.open_raw_tx(inst)
        print('Raw Sender opened: {0}'.format(friendlyName))
        
        if devTx == None:
            sys.exit(-1)

        payloadData = ''
        for j in range(0, payload):
            payloadData += '{0:02X}'.format(j & 0xFF)

        frameId = 0  # only first seq=0 has no timestamp, all following will use hw timestamp from previous send result
        hwTimeStamp = (0,0)
        print('Sending data:')
        print('Payload: {0}'.format(payload))
        print('VLAN-Tag:{0:04X} ({1}/{2})'.format(VLAN_TAG, VLAN_PCP, VLAN_ID))
        print('Press <Ctrl-C> to stop/exit')
        try:
            while 1:

                if VLAN_TAG<0:
                # no VLAN, use LLC default protocol for correct decoding in wireshark
                    sendData = 'FFFFFFFFFFFF' + macSrc + '{0:04X}'.format(payload+18) + '1234' + '{0:08X}'.format(frameId) + '{0:016X}'.format(hwTimeStamp[0]) + '{0:08X}'.format(hwTimeStamp[1]) + payloadData
                else:
                # Single-VLAN on VLAN=0x01 using LLC protocol
                    sendData = 'FFFFFFFFFFFFFC0011223344' + '8100{0:04X}'.format(VLAN_TAG) + '{0:04X}'.format(payload+18) + '1234' + '{0:08X}'.format(frameId) + '{0:016X}'.format(hwTimeStamp[0]) + '{0:08X}'.format(hwTimeStamp[1]) + payloadData
                
                # Double-Tagged VLAN on VLAN-Outer=0x07 and VLAN-Inner=0x15 using LLC protocol
                #sendData = 'FFFFFFFFFFFFFC0011223344' + '88A8000781000015' + '{0:04X}'.format(len(payloadData)+32) + '{0:08X}'.format(frameId) + '{0:016X}'.format(hwTimeStamp[0]) + '{0:08X}'.format(hwTimeStamp[1]) + payloadData
                
                # Trig-Out Frame on 0x88F7 (gPTP EtherType to activate Trig-Out on Raw-Adapter receiving this EtherType) 
                # sendData = 'FFFFFFFFFFFF' + macSrc + '88F7' + '{0:08X}'.format(frameId) + '{0:016X}'.format(hwTimeStamp[0]) + '{0:08X}'.format(hwTimeStamp[1]) + payloadData                
                
                binData = binascii.a2b_hex(sendData)

                res = broadway_api.wait_for_tx(devTx, 1000)
                if not res:
                    print('waitfor tx failed!')

                frameId = broadway_api.submit_raw_frame(devTx, binData)

                res = broadway_api.send_raw_frames(devTx)
            
                res = broadway_api.wait_for_raw_tx_completion(devTx, frameId, 1000)

                hwTimeStamp = broadway_api.get_raw_tx_timestamp(devTx, frameId)

                time.sleep(Delay)
                    
        except KeyboardInterrupt:
            base.close(devTx)
            

if __name__ == "__main__":
    main(sys.argv)
