#
# This file is part of python sample package for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2019 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import sys, os
import platform
import time, datetime
import json
import operator

from tkinter import *
from tkinter import ttk, filedialog, simpledialog

from broadway2 import Broadway, SpiAccess, SmiAccess, SJAXAccess

AppVersion = '1.1.0'    # 2020-10-13 updated for new SJA1110.B1


FontNormal = ('Arial', 10)
FontLarge = ('Arial', 12)
FontHuge = ('Arial', 18)


class App:
    
    def __init__(self, width=0, height=0):
        
        
        self.root = Tk()

        self.root.geometry
        self.root.grid(widthInc=width,heightInc=height)

        #Root-Window Apperace

        self.root.configure(background='#8899AA')
        self.root.title('FC401 - Tool GUI ' + AppVersion)
        self.root.resizable(width=FALSE, height=FALSE)

        #Call Function to Create the Slave Widgets

        self.createWindow()

        #Event Bind

        self.root.bind('<Destroy>',self.shutdown)

        
        self.base = Broadway() # create adapter singleton
        
        self.adapterDict = {}
        
        self.base.set_notify(self.notify_adapter_changed)

        self.twinkleActive = False  # state for twinkle on selected adapter

        #Mainloop
        self.root.mainloop()

    def createWindow(self):
        """
        ----------------------------------------------------------------
         createWindow(self)

          Function: Build the Slave Widgets of the Root  - Window
        -----------------------------------------------------------------
        """
       
        #Adapter - Label

        self.topFrame = Frame(self.root)
        self.topFrame.configure(background='#000000')
        self.topFrame.grid(column=0,row=0,sticky=W)

        lb = Label(self.topFrame, font = FontNormal, text = 'Connected SMI/SPI Adapters: ')
        lb.configure(background='#FCFCFC')
        lb.grid(column=0,row=0,sticky=W)

        
        self.bodyFrame = Frame(self.root)
        self.bodyFrame.grid(column=0,row=1,sticky=W)

        self.statusFrame = Frame(self.root)
        self.statusFrame.configure(background='#000000')
        self.statusFrame.grid(column=0,row=2,sticky=W)
        

        tree = self.tree = ttk.Treeview(self.bodyFrame, height=10)

        treeYScroll = ttk.Scrollbar(self.bodyFrame, orient="vertical",command=tree.yview)
        tree.configure(yscrollcommand=treeYScroll.set)

        tree.bind('<<TreeviewSelect>>', self.tree_selected)


        tree['columns']=('inst', 'fn', 'fw', 'to')
        tree.column(0, width=60 )
        tree.column('inst', width=65 )
        tree.column("fn", width=120)
        tree.column("to", width=90)
        tree.column("fw", width=110)
        tree.heading("inst", text='Instance')
        tree.heading('fn', text='Friendly-Name')
        tree.heading('fw', text='FW-Version')
        tree.heading('to', text='SMI / SPI')

        # create base dirs for supported broadway2 adapters: FC611, FC602 and FC612
       
        self.idFC401 = tree.insert("", 'end', 'FC401', text='FC401')
        

        tree.grid(column=0,row=1,sticky=W)
        treeYScroll.grid(column=2,row=1)

        lb = Label(self.statusFrame, font = FontNormal, text = 'Details: ')
        lb.configure(background='#FCFCFC')
        lb.grid(column=0,row=2,sticky=W)

        self.ToolDetails = StringVar()
        self.lbToolDetails = Label(self.statusFrame, font = FontNormal, textvariable=self.ToolDetails)
        self.lbToolDetails.configure(background='#FCFCFC')
        self.lbToolDetails.grid(column=1, row=2, sticky=W)

    # EVENT Handler
    def shutdown(self, event):
        """close all open instances to clean-up"""
        pass

    def notify_adapter_changed(self, event):
        self.update_gui()

    def get_tool_details(self, a):
        spiRes = ' - '
        smiRes = ' - '

        smi = SmiAccess()
        
        smi.connect(a.Device)
        
        chk = smi.read_reg(0, 0)
        if chk >= 0:
            smiRes = 'OK'


        spi = SpiAccess(a.Device)

        sjaAccess = SJAXAccess(spi, 0)
        
        chk = sjaAccess.read_reg(0)
        if chk >= 0 and chk < 0xFFFFFFFF:
            spiRes = 'OK'

        return (smiRes, spiRes)

    def get_smi_details(self, a):
        res = 'N/A'
        
        smi = SmiAccess()
        
        smi.connect(a.Device)
        res = smi.detect()

        resString = ''
        for phy in res:
            resString += '{0},'.format(phy[0])

        if len(resString) > 0:
            resString = resString[0:-1]
        return resString

    def get_spi_details(self, a):
        res = 'N/A'

        spi = SpiAccess(a.Device)

        sjaAccess = SJAXAccess(spi, 0)
        
        spi.set_protocol(0)

        devId1 = sjaAccess.read_reg(0)
        spi.set_protocol(1)
        devId2 = sjaAccess.read_reg(0)
        if devId1 >= 0 and devId1 < 0xFFFFFFFF:
            res = 'Prot0: {0:08X} Prot1: {1:08X}'.format(devId1, devId2)

        return res


    def update_gui(self):

        for inst in self.adapterDict:
            dev = self.adapterDict[inst].Device
            self.base.close(dev)
         
        self.adapterDict = {}

        for item in self.tree.get_children(self.idFC401):
            self.tree.delete(item)

        aList = self.base.get_tool_adapters()

        for a in aList:

            inst, friendlyName = a

            adapter = self.base.get_adapter(inst)

            dev = self.base.open_control(inst)
            adapter.connect(dev)
            self.adapterDict[adapter.Inst] = adapter


            smiSpiResult = self.get_tool_details(adapter)

            resStr = smiSpiResult[0] + ' / ' + smiSpiResult[1]

            colValues = ['{0}'.format(adapter.Inst), adapter.FriendlyName, adapter.FWRevision, resStr]
            tagsValue = str(adapter.Inst)

            # add new entry under FC401-tree node
            self.tree.insert(self.idFC401, 'end', text='FC401-{0}'.format(adapter.Inst), values=colValues, tags=tagsValue)

    def shutdown(self, event):
        """close all open instances to clean-up"""
        for inst in self.adapterDict:
            a = self.adapterDict[inst]
            print('closing: {0}'.format(a.FriendlyName))
            self.base.close(a.Device)
        
        self.adapterDict = {}

    def tree_selected(self, event):
        selected = self.tree.item(self.tree.focus())
        try:
            inst = selected['values'][0]
            a = self.adapterDict[inst]
            self.twinkle(a)
            smiResult = self.get_smi_details(a)
            spiResult = self.get_spi_details(a)
            self.ToolDetails.set('SMI: {0:35}   SPI: {1:40}'.format(smiResult, spiResult))
        except:
            pass
        
    def twinkle(self, a):
        if self.twinkleActive:
            a.twinkle_stop()
            self.twinkleActive = False
        else:
            a.twinkle_start(3)
            self.twinkleActive = True
            self.root.after(2000, self.twinkle, a)


def main(args):

    # will never return until gui exits
    app = App(150, 50)

if __name__ == "__main__":
    main(sys.argv)
