# raw_receiver.py
#
# This file is part of python wrapper for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2018 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import threading, queue

from broadway2 import Broadway, broadway_api

class RawReceiverThread(threading.Thread):
    def __init__(self, dev, q, e):
        threading.Thread.__init__(self) 
        
        self.dev = dev
        self.outQ = q
        self.stopEvent = e

    def enable(self):
        res = broadway_api.enable_raw_rx(self.dev)

    def disable(self):
        res = broadway_api.disable_raw_rx(self.dev)


    def run(self):
        while True:
            if self.stopEvent.wait(0):
                break

            res = broadway_api.wait_raw_rx_frame(self.dev, 100)
            if res:
                
                res = broadway_api.get_raw_rx_frame(self.dev)
                if res != None:
                    self.outQ.put(res)


class RawReceiver:
    def __init__(self, dev):
        
        self.dev = dev

    def enable(self):
        res = broadway_api.enable_raw_rx(self.dev)
        return res

    def disable(self):
        res = broadway_api.disable_raw_rx(self.dev)
        return res


    def read_block(self, numFrames, timeout):
        result = []
        res = broadway_api.wait_raw_rx_frame(self.dev, timeout)
        if res:
            cnt = numFrames
            while cnt > 0:
                res = broadway_api.get_raw_rx_frame(self.dev)
                if res != None:
                    result.append(res)
                    cnt -= 1
                else:
                    break
        return result

